<?php
include __DIR__."/info.php";

function post_to_mastodon($instance_url, $access_token, $content, $image_ids = NULL) {
    $post_data = array(
            "status" => $content
        );
    
    if (!empty($image_ids)) {
        $post_data["media_ids"] = $image_ids;
    }
    
    $ch = curl_init("https://".$instance_url."/api/v1/statuses");
    
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: Bearer ".$access_token));
    curl_setopt($ch, CURLOPT_POSTFIELDS, preg_replace('/%5B[0-9]+%5D/simU', '%5B%5D', http_build_query($post_data)));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    
    return json_decode(curl_exec($ch), TRUE);
}

function upload_image_to_mastodon($instance_url, $access_token, $image_path) {
    $ch = curl_init("https://".$instance_url."/api/v1/media");
    $cf = curl_file_create($image_path);
    
    $post_data = array(
            "file" => $cf
        );
    
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: Bearer ".$access_token));
    curl_setopt($ch, CURLOPT_POSTFIELDS, $post_data);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    
    $res_json = json_decode(curl_exec($ch), TRUE);
    return $res_json["id"];
}

function create_twitter_signature($method, $api_url, $oauth_params, $consumer_secret, $access_token_secret = NULL) {
    ksort($oauth_params);
    $data = rawurlencode($method)."&".rawurlencode($api_url)."&".rawurlencode(str_replace(array("+", "%7E"), array("%20", "~"), http_build_query($oauth_params)));
    
    $key = rawurlencode($consumer_secret)."&";
    if (!empty($access_token_secret)) {
        $key .= rawurlencode($access_token_secret);
    }
    
    return base64_encode(hash_hmac("SHA1", $data, $key, TRUE));
}

function post_to_twitter($consumer_key, $consumer_secret, $access_token, $access_token_secret, $content, $image_ids = NULL) {
    $post_params = array(
            "status" => $content,
            "oauth_token" => $access_token,
            "oauth_consumer_key" => $consumer_key,
            "oauth_signature_method" => "HMAC-SHA1",
            "oauth_timestamp" => time(),
            "oauth_nonce" => microtime(),
            "oauth_version" => "1.0",
        );
    
    if (!empty($image_ids)) {
        $image_ids_csv = implode(",", $image_ids);
        
        $post_params["media_ids"] = $image_ids_csv;
        $context_content = http_build_query(array("status" => $content, "media_ids" => $image_ids_csv));
    } else {
        $context_content = http_build_query(array("status" => $content));
    }
    
    $update_url = "https://api.twitter.com/1.1/statuses/update.json";
    $post_params["oauth_signature"] = create_twitter_signature("POST", $update_url, $post_params, $consumer_secret, $access_token_secret);
    
    $context = array(
            "http" => array(
                    "method" => "POST",
                    "header" => implode("rn", array(
                            "Authorization: OAuth ".http_build_query($post_params, "", ","),
                            "Content-Type: application/x-www-form-urlencoded",
                            "Content-Length: ".strlen($context_content)
                        )),
                    "content" => $context_content,
                ),
        );
    
    return json_decode(file_get_contents($update_url, FALSE, stream_context_create($context)), TRUE);
}

function upload_image_to_twitter($consumer_key, $consumer_secret, $access_token, $access_token_secret, $image_path) {
    $oauth_params = array(
            "oauth_token" => $access_token,
            "oauth_consumer_key" => $consumer_key,
            "oauth_signature_method" => "HMAC-SHA1",
            "oauth_timestamp" => time(),
            "oauth_nonce" => microtime(),
            "oauth_version" => "1.0",
        );
    
    $upload_url = "https://upload.twitter.com/1.1/media/upload.json";
    $oauth_params["oauth_signature"] = create_twitter_signature("POST", $upload_url, $oauth_params, $consumer_secret, $access_token_secret);
    
    $context_content = array("media" => file_get_contents($image_path));
    $context = array(
            "http" => array(
                    "method" => "POST",
                    "header" => array(
                            "Authorization: OAuth ".http_build_query($oauth_params, "", ","),
                            "Content-Type: multipart/form-data;"
                        ),
                    "content" => $context_content,
                ),
        );
    
    $ch = curl_init($upload_url);
    
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch ,CURLOPT_HTTPHEADER, $context["http"]["header"]);
    curl_setopt($ch ,CURLOPT_POSTFIELDS, $context["http"]["content"]);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    
    $res_json = json_decode(curl_exec($ch), TRUE);
    return $res_json["media_id"];
}

function get_mastodon_client_data($instance_url) {
    $post_data = array(
            "client_name" => APPLICATION_NAME,
            "redirect_uris" => get_application_url()."register_token/mastodon.php",
            "scopes" => "read write follow",
            "website" => get_application_url()
        );
    
    $ch = curl_init("https://".$instance_url."/api/v1/apps");
    
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_data));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    
    return json_decode(curl_exec($ch), TRUE);
}

function get_mastodon_token($instance_url, $client_id, $client_secret, $code) {
    $post_data = array(
            "grant_type" => "authorization_code",
            "redirect_uri" => get_application_url()."register_token/mastodon.php",
            "client_id" => $client_id,
            "client_secret" => $client_secret,
            "code" => $code
        );
    
    $ch = curl_init("https://".$instance_url."/oauth/token");
    
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_data));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    
    $response_data = json_decode(curl_exec($ch), TRUE);
    
    return $response_data["access_token"];
}

function get_twitter_oauth_token($consumer_key, $consumer_secret) {
    $oauth_params = array(
            "oauth_callback" => get_application_url()."register_token/twitter.php",
            "oauth_consumer_key" => $consumer_key,
            "oauth_nonce" => md5(get_application_url().microtime()),
            "oauth_timestamp" => time(),
            "oauth_version" => "1.0",
            "oauth_signature_method" => "HMAC-SHA1",
        );
    $api_token_url = "https://api.twitter.com/oauth/request_token";
    $oauth_params["oauth_signature"] = create_twitter_signature("GET", $api_token_url, $oauth_params, $consumer_secret);
    
    $twitter_res = file_get_contents($api_token_url."?".http_build_query($oauth_params));
    parse_str($twitter_res, $token_data);
    
    if (empty($token_data)) {
        return FALSE;
    }
    return $token_data["oauth_token"];
}

function get_twitter_access_token($consumer_key, $consumer_secret, $oauth_token, $oauth_verifier) {
    $oauth_params = array(
        "oauth_consumer_key" => $consumer_key,
        "oauth_nonce" => md5(get_application_url().microtime()),
        "oauth_signature_method" => "HMAC-SHA1",
        "oauth_timestamp" => time(),
        "oauth_token" => $oauth_token,
        "oauth_verifier" => $oauth_verifier,
        "oauth_version" => "1.0",
    );
    
    $access_token_url = "https://api.twitter.com/oauth/access_token";
    $oauth_params["oauth_signature"] = create_twitter_signature("POST", $access_token_url, $oauth_params, $consumer_secret);
    
    $context_content = http_build_query(array());
    $context = array(
            "http" => array(
                    "method" => "POST",
                    "header" => implode("rn", array(
                            "Authorization: OAuth ".http_build_query($oauth_params, "", ","),
                            "Content-Type: application/x-www-form-urlencoded",
                            "Content-Length: ".strlen($context_content)
                        )),
                    "content" => $context_content,
                ),
        );
    
    $twitter_res = file_get_contents($access_token_url, FALSE, stream_context_create($context));
    parse_str($twitter_res, $token_data);
    
    if (empty($token_data)) {
        return FALSE;
    }
    
    return $token_data;
}

function resize_image($image_file_name, $new_file_name, $image_transformation) {
    $orientation = 0;
    
    switch (exif_imagetype($image_file_name)) {
        case IMAGETYPE_PNG:
            $format = "png";
            $img = imagecreatefrompng($image_file_name);
            break;
        case IMAGETYPE_JPEG:
            $format = "jpeg";
            $img = imagecreatefromjpeg($image_file_name);
            $exif = exif_read_data($image_file_name);
            if (isset($exif["Orientation"])) {
                switch ($exif["Orientation"]) {
                    case 3:
                        $orientation = 180;
                        break;
                    case 6:
                        $orientation = 270;
                        break;
                    case 8:
                        $orientation = 90;
                        break;
                }
            }
            break;
        case IMAGETYPE_GIF:
            $format = "gif";
            $img = imagecreatefromgif($image_file_name);
            break;
        default:
            print "指定された画像ファイルは対応していない形式です。";
            return FALSE;
    }
    
    if (!empty($image_transformation["rotate"])) {
        $orientation = ($orientation + (360 - $image_transformation["rotate"])) % 360;
    }
    
    if ($orientation != 0) {
        if ($format == "png") {
            $img = imagerotate($img, $orientation, imagecolorallocatealpha($img, 0, 0, 0, 127));
        } else {
            $img = imagerotate($img, $orientation, imagecolorallocate($img, 0xFF, 0xFF, 0xFF));
        }
    }
    
    $img_x = imagesx($img);
    $img_y = imagesy($img);
    
    if (!empty($image_transformation["trimming"])
            && $image_transformation["trimming"]["x1"] != $image_transformation["trimming"]["x0"]
            && $image_transformation["trimming"]["y1"] != $image_transformation["trimming"]["y0"]
            && (abs($image_transformation["trimming"]["x1"] - $image_transformation["trimming"]["x0"]) != 100 || abs($image_transformation["trimming"]["y1"] - $image_transformation["trimming"]["y0"]) != 100)) {
        $x0 = round($img_x * $image_transformation["trimming"]["x0"] / 100);
        $y0 = round($img_y * $image_transformation["trimming"]["y0"] / 100);
        $x1 = round($img_x * $image_transformation["trimming"]["x1"] / 100);
        $y1 = round($img_y * $image_transformation["trimming"]["y1"] / 100);
        
        if ($x0 > $x1) {
            $tmp = $x0;
            $x0 = $x1;
            $x1 = $tmp;
        }
        if ($y0 > $y1) {
            $tmp = $y0;
            $y0 = $y1;
            $y1 = $tmp;
        }
        
        $img_w = $x1 - $x0;
        $img_h = $y1 - $y0;
        if ($img_w > MAX_IMAGE_WIDTH || $img_h > MAX_IMAGE_HEIGHT) {
            $x_hiritsu = $img_w / MAX_IMAGE_WIDTH;
            $y_hiritsu = $img_h / MAX_IMAGE_HEIGHT;
            
            if ($x_hiritsu >= $y_hiritsu) {
                $x_after = MAX_IMAGE_WIDTH;
                $y_after = round($img_h / $x_hiritsu);
            }else{
                $x_after = round($img_w / $y_hiritsu);
                $y_after = MAX_IMAGE_HEIGHT;
            }
        } else {
            $x_after = $img_w;
            $y_after = $img_h;
        }
        $img_after = imagecreatetruecolor($x_after, $y_after);
        
        switch($format){
            case "jpeg":
            case "gif":
                imagefilledrectangle($img_after, 0, 0, $x_after, $y_after, imagecolorallocate($img_after, 0xFF, 0xFF, 0xFF));
                break;
            case "png":
                imagealphablending($img_after,FALSE);
                imagesavealpha($img_after,TRUE);
                break;
        }
        
        imagecopyresampled($img_after, $img, 0, 0, $x0, $y0, $x_after, $y_after, $img_w, $img_h);
    } else {
        if ($img_x > MAX_IMAGE_WIDTH || $img_y > MAX_IMAGE_HEIGHT) {
            $x_hiritsu = $img_x / MAX_IMAGE_WIDTH;
            $y_hiritsu = $img_y / MAX_IMAGE_HEIGHT;
            
            if($x_hiritsu >= $y_hiritsu){
                $x_after = MAX_IMAGE_WIDTH;
                $y_after = round($img_y / $x_hiritsu);
            }else{
                $x_after = round($img_x / $y_hiritsu);
                $y_after = MAX_IMAGE_HEIGHT;
            }
            
            $img_after = imagecreatetruecolor($x_after, $y_after);
            
            switch($format){
                case "jpeg":
                case "gif":
                    imagefilledrectangle($img_after, 0, 0, $x_after, $y_after, imagecolorallocate($img_after, 0xFF, 0xFF, 0xFF));
                    break;
                case "png":
                    imagealphablending($img_after,FALSE);
                    imagesavealpha($img_after,TRUE);
                    break;
            }
            
            imagecopyresampled($img_after, $img, 0, 0, 0, 0, $x_after, $y_after, $img_x, $img_y);
        } else {
            if ($orientation == 0) {
                rename($image_file_name, $new_file_name);
                return;
            }
        
            $img_after = $img;
        }
    }
    
    switch($format){
        case "jpeg":
        imagejpeg($img_after, $new_file_name, JPEG_COMPRESSION_QUALITY);
            break;
        case "png":
            imagepng($img_after, $new_file_name);
            break;
        case "gif":
            imagegif($img_after, $new_file_name);
            break;
    }
}